/* BdfServer - Copyright (c) 2024-2025 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.instruction;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.exportation.table.FicheTableParameters;
import fr.exemole.bdfserver.api.instruction.BdfParameters;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.managers.TableExportManager;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.exportation.table.FicheTableParametersBuilder;
import java.text.ParseException;
import java.util.LinkedHashMap;
import java.util.Map;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.exportation.access.AccessDef;
import net.fichotheque.exportation.balayage.BalayageContentDescription;
import net.fichotheque.exportation.balayage.BalayageDescription;
import net.fichotheque.exportation.scrutari.ScrutariExportDef;
import net.fichotheque.exportation.sql.SqlExportDef;
import net.fichotheque.exportation.table.TableExportConstants;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.permission.PermissionSummary;
import net.fichotheque.selection.SelectionDef;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.BalayageUtils;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.tableparser.CsvParameters;
import net.mapeadores.util.text.tableparser.SheetCopyParameters;
import net.mapeadores.util.text.tableparser.TableParser;


/**
 *
 * @author Vincent Calame
 */
public class RequestHandler implements BdfParameters {

    private final BdfParameters bdfParameters;
    private final RequestMap requestMap;
    private Map<String, String> storeMap = null;
    private boolean store = false;
    private Map<String, String> defaultMap = null;

    public RequestHandler(OutputParameters outputParameters) {
        this(outputParameters, outputParameters.getRequestMap());
    }

    public RequestHandler(BdfParameters bdfParameters, RequestMap requestMap) {
        this.bdfParameters = bdfParameters;
        this.requestMap = requestMap;
    }

    public RequestHandler setDefaultMap(Map<String, String> defaultMap) {
        this.defaultMap = defaultMap;
        return this;
    }

    public RequestHandler disableStore() {
        this.store = false;
        return this;
    }

    public RequestHandler enableStore() {
        this.store = true;
        if (storeMap == null) {
            this.storeMap = new LinkedHashMap<String, String>();
        }
        return this;
    }

    @Override
    public BdfServer getBdfServer() {
        return bdfParameters.getBdfServer();
    }

    @Override
    public BdfUser getBdfUser() {
        return bdfParameters.getBdfUser();
    }

    @Override
    public PermissionSummary getPermissionSummary() {
        return bdfParameters.getPermissionSummary();
    }

    public boolean hasParameter(String paramName) {
        return (requestMap.getParameter(paramName) != null);
    }

    public boolean isTrue(String paramName) {
        String paramValue = requestMap.getTrimedParameter(paramName);
        if (paramValue == null) {
            return false;
        }
        store(paramName, paramValue);
        return StringUtils.isTrue(paramValue);
    }

    public String[] getParameterValues(String name) {
        return requestMap.getParameterValues(name);
    }

    public String getParameter(String paramName, boolean mandatory) throws ErrorMessageException {
        if (mandatory) {
            return getMandatoryParameter(paramName);
        } else {
            return getTrimedParameter(paramName);
        }
    }

    public String getMandatoryParameter(String paramName) throws ErrorMessageException {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            throw BdfErrors.emptyMandatoryParameter(paramName);
        }
        store(paramName, paramValue);
        return paramValue;
    }

    public String getTrimedParameter(String paramName) throws ErrorMessageException {
        return getTrimedParameter(paramName, "");
    }

    public String getTrimedParameter(String paramName, String defaultValue) throws ErrorMessageException {
        String paramValue = requestMap.getTrimedParameter(paramName);
        if (paramValue.isEmpty()) {
            if (defaultMap != null) {
                String value = defaultMap.get(paramName);
                if ((value != null) && (!value.isEmpty())) {
                    return value;
                }
            }
            return defaultValue;
        } else {
            store(paramName, paramValue);
            return paramValue;
        }
    }

    public String[] getTokens(String paramName) throws ErrorMessageException {
        String[] tokens = requestMap.getParameterValues(paramName);
        if (tokens != null) {
            String compilation = StringUtils.implode(tokens, ';');
            store(paramName, compilation);
            return tokens;
        }
        if (defaultMap != null) {
            String value = defaultMap.get(paramName);
            if ((value != null) && (!value.isEmpty())) {
                return StringUtils.getTokens(value, ';', StringUtils.EMPTY_EXCLUDE);
            }
        }
        return StringUtils.EMPTY_STRINGARRAY;
    }

    public FicheMeta getMandatoryFicheMeta() throws ErrorMessageException {
        return (FicheMeta) getMandatorySubsetItem(SubsetKey.CATEGORY_CORPUS);
    }

    public Motcle getMandatoryMotcle() throws ErrorMessageException {
        String idString = requestMap.getParameter(InteractionConstants.IDMTCL_PARAMNAME);
        if (idString != null) {
            return getMandatoryMotcle(InteractionConstants.IDMTCL_PARAMNAME);
        }
        String idalpha = requestMap.getTrimedParameter(InteractionConstants.IDALPHA_PARAMNAME);
        if (!idalpha.isEmpty()) {
            Thesaurus thesaurus = getMandatoryThesaurus();
            if (thesaurus.isIdalphaType()) {
                Motcle motcle = thesaurus.getMotcleByIdalpha(idalpha);
                if (motcle != null) {
                    store(InteractionConstants.IDALPHA_PARAMNAME, idalpha);
                    return motcle;
                } else {
                    throw BdfErrors.wrongParameterValue(InteractionConstants.IDALPHA_PARAMNAME, idalpha);
                }
            }
        }
        return (Motcle) getMandatorySubsetItem(SubsetKey.CATEGORY_THESAURUS);
    }

    public Motcle getMandatoryMotcle(String paramName) throws ErrorMessageException {
        String idString = requestMap.getParameter(paramName);
        Thesaurus thesaurus = getMandatoryThesaurus();
        Motcle motcle = null;
        try {
            motcle = getMotcle(thesaurus, idString);
        } catch (NumberFormatException nfe) {
            throw BdfErrors.wrongParameterValue(paramName, idString);
        }
        if (motcle == null) {
            throw BdfErrors.unknownParameterValue(paramName, idString);
        }
        store(paramName, idString);
        return motcle;
    }

    public Motcle getOptionnalMotcle(Thesaurus thesaurus) throws ErrorMessageException {
        String idmtclValue = requestMap.getParameter(InteractionConstants.IDMTCL_PARAMNAME);
        if (idmtclValue == null) {
            return (Motcle) getOptionnalSubsetItem(thesaurus);
        }
        try {
            Motcle motcle = getMotcle(thesaurus, idmtclValue);
            store(InteractionConstants.IDMTCL_PARAMNAME, idmtclValue);
            return motcle;
        } catch (NumberFormatException nfe) {
            throw BdfErrors.wrongParameterValue(InteractionConstants.IDMTCL_PARAMNAME, idmtclValue);
        }
    }

    public Corpus getMandatoryCorpus() throws ErrorMessageException {
        return (Corpus) getMandatorySubset(SubsetKey.CATEGORY_CORPUS);
    }

    public Redacteur getMandatoryRedacteur() throws ErrorMessageException {
        return (Redacteur) getMandatorySubsetItem(SubsetKey.CATEGORY_SPHERE);
    }

    public Subset getMandatorySubset(short subsetCategory) throws ErrorMessageException {
        return getMandatorySubset(subsetCategory, SubsetKey.categoryToString(subsetCategory));
    }

    public Subset getMandatorySubset(short subsetCategory, String paramName) throws ErrorMessageException {
        String paramValue = getMandatoryParameter(paramName);
        try {
            SubsetKey subsetKey = SubsetKey.parse(subsetCategory, paramValue);
            Subset subset = getFichotheque().getSubset(subsetKey);
            if (subset == null) {
                throw BdfErrors.unknownParameterValue(paramName, paramValue);
            }
            return subset;
        } catch (ParseException pe) {
            throw BdfErrors.wrongParameterValue(paramName, paramValue);
        }
    }

    public Subset getMandatorySubset() throws ErrorMessageException {
        return getMandatorySubset(InteractionConstants.SUBSET_PARAMNAME);
    }

    public Subset getMandatorySubset(String paramName) throws ErrorMessageException {
        String paramValue = getMandatoryParameter(paramName);
        try {
            SubsetKey subsetKey = SubsetKey.parse(paramValue);
            Subset subset = getFichotheque().getSubset(subsetKey);
            if (subset == null) {
                throw BdfErrors.unknownParameterValue(paramName, paramValue);
            }
            return subset;
        } catch (ParseException pe) {
            throw BdfErrors.wrongParameterValue(paramName, paramValue);
        }
    }

    public ScrutariExportDef getMandatoryScrutariExportDef() throws ErrorMessageException {
        String scrutariexportString = getMandatoryParameter(InteractionConstants.SCRUTARIEXPORT_PARAMNAME);
        ScrutariExportDef scrutariExportDef = getBdfServer().getScrutariExportManager().getScrutariExportDef(scrutariexportString);
        if (scrutariExportDef == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.SCRUTARIEXPORT_PARAMNAME, scrutariexportString);
        }
        return scrutariExportDef;
    }

    public SelectionDef getMandatorySelectionDef() throws ErrorMessageException {
        String param_selection = getMandatoryParameter(InteractionConstants.SELECTION_PARAMNAME);
        SelectionDef selectionDef = getBdfServer().getSelectionManager().getSelectionDef(param_selection);
        if (selectionDef == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.SELECTION_PARAMNAME, param_selection);
        }
        return selectionDef;
    }

    public SqlExportDef getMandatorySqlExportDef() throws ErrorMessageException {
        String param_sqlexport = getMandatoryParameter(InteractionConstants.SQLEXPORT_PARAMNAME);
        SqlExportDef sqlExportDef = getBdfServer().getSqlExportManager().getSqlExportDef(param_sqlexport);
        if (sqlExportDef == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.SQLEXPORT_PARAMNAME, param_sqlexport);
        }
        return sqlExportDef;
    }

    public AccessDef getMandatoryAccessDef() throws ErrorMessageException {
        String param_access = getMandatoryParameter(InteractionConstants.ACCESS_PARAMNAME);
        AccessDef accessDef = getBdfServer().getAccessManager().getAccessDef(param_access);
        if (accessDef == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.ACCESS_PARAMNAME, param_access);
        }
        return accessDef;
    }

    public String getMandatoryPath() throws ErrorMessageException {
        String s = getMandatoryParameter(InteractionConstants.PATH_PARAMNAME);
        String path = s.trim();
        if (path.length() == 0) {
            throw BdfErrors.wrongParameterValue(InteractionConstants.PATH_PARAMNAME, s);
        }
        return path;
    }

    public BalayageDescription getMandatoryBalayageDescription() throws ErrorMessageException {
        String param_balayage = getMandatoryParameter(InteractionConstants.BALAYAGE_PARAMNAME);
        BalayageDescription balayageDescription = getBdfServer().getBalayageManager().getBalayage(param_balayage);
        if (balayageDescription == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.BALAYAGE_PARAMNAME, param_balayage);
        }
        return balayageDescription;
    }

    public BalayageContentDescription getMandatoryBalayageContentDescription(BalayageDescription balayageDescription) throws ErrorMessageException {
        String contentPath = getMandatoryPath();
        BalayageContentDescription description = BalayageUtils.getBalayageContentDescription(balayageDescription, contentPath);
        if (description == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.PATH_PARAMNAME, contentPath);
        }
        return description;
    }

    public TableExportDescription getMandatoryTableExportDescription() throws ErrorMessageException {
        TableExportManager tableExportManager = getBdfServer().getTableExportManager();
        String tableExportName = getMandatoryParameter(InteractionConstants.TABLEEXPORT_PARAMNAME);
        for (TableExportDescription tableExportDescription : tableExportManager.getTableExportDescriptionList()) {
            if (tableExportDescription.getName().equals(tableExportName)) {
                return tableExportDescription;
            }
        }
        throw BdfErrors.unknownParameterValue(InteractionConstants.TABLEEXPORT_PARAMNAME, tableExportName);
    }

    public Thesaurus getMandatoryThesaurus() throws ErrorMessageException {
        return (Thesaurus) getMandatorySubset(SubsetKey.CATEGORY_THESAURUS);
    }

    public String getHeaderType() throws ErrorMessageException {
        String headerTypeValue = getTrimedParameter(InteractionConstants.HEADERTYPE_PARAMNAME);
        if (headerTypeValue.isEmpty()) {
            return TableExportConstants.COLUMNTITLE_HEADER;
        }
        try {
            return TableExportConstants.checkHeaderType(headerTypeValue);
        } catch (IllegalArgumentException iae) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.HEADERTYPE_PARAMNAME, headerTypeValue);
        }
    }

    public String getMode() throws ErrorMessageException {
        String mode = requestMap.getTrimedParameter(InteractionConstants.MODE_PARAMNAME);
        if (mode.isEmpty()) {
            return "";
        }
        try {
            StringUtils.checkTechnicalName(mode, false);
        } catch (ParseException pe) {
            throw BdfErrors.error("_ error.wrong.mode", mode);
        }
        store(InteractionConstants.MODE_PARAMNAME, mode);
        return mode;
    }

    public int getMandatoryId() throws ErrorMessageException {
        int id = getMandatoryIntegerParameter(InteractionConstants.ID_PARAMNAME);
        if (id < 1) {
            throw BdfErrors.wrongParameterValue(InteractionConstants.ID_PARAMNAME, requestMap.getParameter(InteractionConstants.ID_PARAMNAME));
        }
        return id;
    }

    public int getMandatoryIntegerParameter(String paramName) throws ErrorMessageException {
        String paramValue = requestMap.getParameter(paramName);
        if (paramValue == null) {
            throw BdfErrors.emptyMandatoryParameter(paramName);
        }
        try {
            int value = Integer.parseInt(paramValue);
            store(paramName, String.valueOf(value));
            return value;
        } catch (NumberFormatException nfe) {
            throw BdfErrors.wrongParameterValue(paramName, paramValue);
        }
    }

    public SubsetItem getOptionnalSubsetItem(Subset subset) throws ErrorMessageException {
        return getOptionnalSubsetItem(subset, InteractionConstants.ID_PARAMNAME);
    }

    public SubsetItem getOptionnalSubsetItem(Subset subset, String paramName) throws ErrorMessageException {
        String idString = requestMap.getParameter(paramName);
        if (idString == null) {
            return null;
        }
        int id = 0;
        try {
            id = Integer.parseInt(idString);
        } catch (NumberFormatException nfe) {
            throw BdfErrors.wrongParameterValue(paramName, idString);
        }
        if (id < 1) {
            throw BdfErrors.wrongParameterValue(paramName, idString);
        }
        SubsetItem subsetItem = subset.getSubsetItemById(id);
        store(paramName, String.valueOf(id));
        return subsetItem;
    }

    public FicheTableParameters getFicheTableParameters() throws ErrorMessageException {
        FicheTableParametersBuilder ficheTableParametersBuilder = new FicheTableParametersBuilder();
        String[] tokens = getTokens(InteractionConstants.WITH_PARAMNAME);
        for (String token : tokens) {
            ficheTableParametersBuilder.putWith(token);
        }
        String patternMode = getTrimedParameter(InteractionConstants.PATTERNMODE_PARAMNAME);
        if (!patternMode.isEmpty()) {
            try {
                ficheTableParametersBuilder.setPatternMode(patternMode);
            } catch (IllegalArgumentException iae) {
                throw new ErrorMessageException("_ error.unknown.patternmode", patternMode);
            }
        }
        return ficheTableParametersBuilder.toFicheTableParameters();
    }

    public TableParser.Parameters getTableParserParameters() throws ErrorMessageException {
        String parseType = getMandatoryParameter(InteractionConstants.PARSETYPE_PARAMNAME);
        String delimParam = getTrimedParameter(InteractionConstants.CSV_DELIMITER_PARAMNAME);
        String quoteParam = getTrimedParameter(InteractionConstants.CSV_QUOTE_PARAMNAME);
        boolean escapedCSV = isTrue(InteractionConstants.CSV_ESCAPED_PARAMNAME);
        TableParser.Parameters params = null;
        if (parseType.equals(TableParser.SHEETCOPY_PARSETYPE)) {
            params = new SheetCopyParameters();
        } else if (parseType.equals(TableParser.CSV_PARSETYPE)) {
            CsvParameters csvParameters = new CsvParameters();
            if (!delimParam.isEmpty()) {
                try {
                    char delim = StringUtils.stringToChar(delimParam);
                    csvParameters.setDelimiter(delim);
                } catch (IllegalArgumentException iae) {
                    throw BdfErrors.wrongParameterValue(InteractionConstants.CSV_DELIMITER_PARAMNAME, delimParam);
                }
            }
            if (!quoteParam.isEmpty()) {
                try {
                    char quote = StringUtils.stringToChar(quoteParam);
                    csvParameters.setQuote(quote);
                } catch (IllegalArgumentException iae) {
                    throw BdfErrors.wrongParameterValue(InteractionConstants.CSV_QUOTE_PARAMNAME, quoteParam);
                }
            }
            csvParameters.setEscapedCSV(escapedCSV);
            params = csvParameters;
        } else {
            throw BdfErrors.wrongParameterValue(InteractionConstants.PARSETYPE_PARAMNAME, parseType);
        }
        return params;
    }

    private SubsetItem getMandatorySubsetItem(short subsetCategory) throws ErrorMessageException {
        int id = getMandatoryId();
        Subset subset = getMandatorySubset(subsetCategory);
        SubsetItem subsetItem = subset.getSubsetItemById(id);
        if (subsetItem == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.ID_PARAMNAME, String.valueOf(id));
        }
        return subsetItem;
    }

    private SubsetItem getMandatorySubsetItem(Subset subset) throws ErrorMessageException {
        int id = getMandatoryId();
        SubsetItem subsetItem = subset.getSubsetItemById(id);
        if (subsetItem == null) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.ID_PARAMNAME, String.valueOf(id));
        }
        return subsetItem;
    }

    private void store(String paramName, String paramValue) {
        if (store) {
            storeMap.put(paramName, paramValue);
        }
    }

    public void store(String storeName) {
        if (storeMap != null) {
            getBdfServer().store(getBdfUser(), storeName, storeMap);
            storeMap.clear();
        }
    }

    public static Motcle getMotcle(Thesaurus thesaurus, String idString) throws NumberFormatException {
        if (thesaurus.isIdalphaType()) {
            Motcle motcle = thesaurus.getMotcleByIdalpha(idString);
            if (motcle != null) {
                return motcle;
            } else {
                try {
                    int motcleid = Integer.parseInt(idString);
                    return thesaurus.getMotcleById(motcleid);
                } catch (NumberFormatException nfe) {
                    return null;
                }
            }
        } else {
            int motcleid = Integer.parseInt(idString);
            return thesaurus.getMotcleById(motcleid);
        }
    }

    public static RequestHandler init(BdfParameters bdfParameters, RequestMap requestMap) {
        return new RequestHandler(bdfParameters, requestMap);
    }

    public static RequestHandler init(OutputParameters outputParameters) {
        return new RequestHandler(outputParameters);
    }

}
